#include "mcs_xml_io.h"
#include <iostream>


mcs_xml_io::mcs_xml_io(string mydoctype)
	: rootdoc(NULL), doctype(mydoctype)
{
	xml_init();
	xml_createrootdoc(mydoctype);  
}

mcs_xml_io::mcs_xml_io(mcs_xml_io& toCopy) 
	: rootdoc(NULL), doctype(toCopy.doctype)
{
	xml_init();
	xml_createrootdoc(doctype);
}

mcs_xml_io::~mcs_xml_io()
{
	if (rootdoc!=NULL)
	{
		delete rootdoc;
		rootdoc = NULL;
	}
	this->xml_terminate();
}
 
void mcs_xml_io::xml_init()
{
	// setzen des aktuellen Gebietsschemas...
	setlocale(LC_ALL,NULL);
	// ...ausgenommen sind Ziffern
	setlocale(LC_NUMERIC,"C");

	try
	{ 
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException& c_roToCatch)
	{
		cout	 << "Fehler bei der Initialisierung:" << c_roToCatch.getMessage() << endl;
		exit(1);
	};
	cout << "XML init correct!" << endl;
}

void mcs_xml_io::xml_terminate()
{
	XMLPlatformUtils::Terminate();
	cout << "XML terminate correct!" << endl;
}

void mcs_xml_io::xml_filewriter(string xml_filename)
{	 
	 DOMDocument *doc=rootdoc;
	 DOMImplementation* impl =  DOMImplementationRegistry::getDOMImplementation(X("LS"));
	 DOMWriter* theSerializer = ((DOMImplementationLS*)impl)->createDOMWriter();
	 
	 if (theSerializer->canSetFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true))
    		theSerializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true);
         
	 XMLFormatTarget *myFileTarget = new LocalFileFormatTarget(X(xml_filename.c_str()));
	 theSerializer->writeNode(myFileTarget, *doc);
	 theSerializer->release();
	 delete myFileTarget;  
}

void mcs_xml_io::xml_createrootdoc(string mydoctype)

{
	if (rootdoc!=NULL)
	{
		delete rootdoc;
		rootdoc = NULL;
	}
	if (mydoctype!="RNAPatterns")
	{
		cout << "Doctype: " << mydoctype << " not supported!" << endl;
		exit(1);
	}
	else
	{
	 	DOMImplementation *pImpl = DOMImplementation::getImplementation();
  		DOMDocumentType* pDoctype = pImpl->createDocumentType( X("RNAPatterns"), NULL, X("RNAPatterns.dtd") );
  		rootdoc = pImpl->createDocument(NULL,X("RNAPatterns"), pDoctype);

		DOMElement* rootElem = rootdoc->getDocumentElement();
 		rootElem->setAttribute(X("version"), X("0.1"));
	    DOMElement*  moleculesElem = rootdoc->createElement(X("molecules"));
		rootElem->appendChild(moleculesElem);
		DOMElement*  patternsElem = rootdoc->createElement(X("patterns"));
		rootElem->appendChild(patternsElem);
	}       
}

void	 mcs_xml_io::xml_append_molecule(Molecule& myMol)
{
	
	 DOMElement* pContentRoot = rootdoc->getDocumentElement();	 
	 DOMTreeWalker *walker = rootdoc->createTreeWalker(pContentRoot, DOMNodeFilter::SHOW_ALL, NULL, true);
    	 DOMNode* pCursor=walker->firstChild();
     if (  pCursor->getNodeType() == DOMNode::ELEMENT_NODE &&
          !XMLString::compareString(pCursor->getNodeName(), X("molecules") )  )
     {
     	if (walker->lastChild()!=NULL) 
     		pCursor = walker->parentNode();
     		
        DOMElement* rootElem=(DOMElement *)pCursor;
                
        DOMElement*  moleculeElem = rootdoc->createElement(X("molecule"));
        moleculeElem->setAttribute(X("id"), X(myMol.getId().c_str()));
        moleculeElem->setIdAttribute(X("id"));  
        DOMElement*  identityElem = rootdoc->createElement(X("identity"));
		moleculeElem->appendChild(identityElem);
		DOMElement*  idnameElem = rootdoc->createElement(X("name"));
		identityElem->appendChild(idnameElem);
		DOMText*    	idnameVal = rootdoc->createTextNode(X(myMol.getId().c_str()));
        idnameElem->appendChild(idnameVal);
			   
		DOMElement*  sequenceElem = rootdoc->createElement(X("sequence"));
		sequenceElem->setAttribute(X("length"),X((biu::Util_String::int2str(myMol.getSequence().length())).c_str()));
		DOMText*		seqVal = rootdoc->createTextNode(X(myMol.getSequence().c_str()));
		sequenceElem->appendChild(seqVal);
		moleculeElem->appendChild(sequenceElem);
			   
		DOMElement*  structureElem = rootdoc->createElement(X("structure"));
        moleculeElem	->appendChild(structureElem);
        DOMElement*  modelElem = rootdoc->createElement(X("model"));
        modelElem		->setAttribute(X("id"),X(myMol.getActualStructureId().c_str()));
        modelElem	->setIdAttribute(X("id"));
        DOMElement*  structdataElem = rootdoc->createElement(X("struct_data"));
        DOMText*		structdataVal = rootdoc->createTextNode(X(myMol.getStructure().c_str()));
        structdataElem->appendChild(structdataVal);
        modelElem->appendChild(structdataElem);
        structureElem->appendChild(modelElem);
        rootElem->appendChild(moleculeElem);
        cout << "1 Molecule added!" << endl;
     }
}

/*void	 mcs_xml_io::xml_append_patterns(PatternMap& myPMap)
{
	DOMElement* pContentRoot = rootdoc->getDocumentElement();	 
	DOMTreeWalker *walker = rootdoc->createTreeWalker(pContentRoot, DOMNodeFilter::SHOW_ALL, NULL, true);
    DOMNode* pCursor=walker->lastChild();
    	
    	if (  pCursor->getNodeType() == DOMNode::ELEMENT_NODE &&
          !XMLString::compareString(pCursor->getNodeName(), X("patterns") )  )
     {
     	if (walker->lastChild()!=NULL) 
     		pCursor = walker->parentNode();
     		    
        for (cpMapIter i=myPMap.getMap().begin();i != myPMap.getMap().end();i++)
        {	
            string lenstr=biu::Util_String::int2str(i->second.getMap().begin()->second.getPat().size());                 		
        		DOMElement* rootElem=(DOMElement *)pCursor;
     			DOMElement*  patternElem = rootdoc->createElement(X("pattern"));
        		patternElem->setAttribute(X("id"), X(i->second.getid().c_str()));
        		patternElem->setAttribute(X("size"), X(lenstr.c_str()));
        		
            for (csPMapIter j=i->second.getMap().begin();j!=i->second.getMap().end();j++)
            {
        		  vector <int> tmp=j->second.getPat();
              string tmpstr=biu::Util_String::intvec2str(tmp,":");   
              tmpstr.resize(tmpstr.length()-1);
            
              DOMElement*  patterndataElem = rootdoc->createElement(X("pattern_data"));
        		  patterndataElem->setAttribute(X("molecule_ref"), X(j->second.getID().getMolRef().c_str()));
        		  patterndataElem->setAttribute(X("model_ref"), X(j->second.getID().getStrRef().c_str()));
        		  DOMElement*  positionsElem = rootdoc->createElement(X("positions"));
        		  DOMText*  positionsVal = rootdoc->createTextNode(X(tmpstr.c_str()));
        		  positionsElem->appendChild(positionsVal);
        		  patterndataElem->appendChild(positionsElem);
        		  patternElem->appendChild(patterndataElem);
            }
        		rootElem->appendChild(patternElem);		
        }
     }
}
*/
void   mcs_xml_io::xml_append_patterns(PatternPairMap& myPMap)
{

   DOMElement* pContentRoot = rootdoc->getDocumentElement();    
   DOMTreeWalker *walker = rootdoc->createTreeWalker(pContentRoot, DOMNodeFilter::SHOW_ALL, NULL, true);
    DOMNode* pCursor=walker->lastChild();
      
      if (  pCursor->getNodeType() == DOMNode::ELEMENT_NODE &&
          !XMLString::compareString(pCursor->getNodeName(), X("patterns") )  )
     {
      if (walker->lastChild()!=NULL) 
         pCursor = walker->parentNode();
             
        for (PatternPairMap::patListCITER i=myPMap.getList().begin();i != myPMap.getList().end();i++)
        {   
            string lenstr=biu::Util_String::int2str((*i)->getFirstPat().getPat().size());                      
            DOMElement* rootElem=(DOMElement *)pCursor;
            DOMElement*  patternElem = rootdoc->createElement(X("pattern"));
            patternElem->setAttribute(X("id"), X((*i)->getId().c_str()));
            patternElem->setAttribute(X("size"), X(lenstr.c_str()));
            
              intVec tmp=(*i)->getFirstPat().getPat();
              string tmpstr=biu::Util_String::intvec2str(tmp,":");   
              tmpstr.resize(tmpstr.length()-1);
            
              DOMElement*  patterndataElem = rootdoc->createElement(X("pattern_data"));
              patterndataElem->setAttribute(X("molecule_ref"), X((*i)->getFirstPat().getID().getMolRef().c_str()));
              patterndataElem->setAttribute(X("model_ref"), X((*i)->getFirstPat().getID().getStrRef().c_str()));
              DOMElement*  positionsElem = rootdoc->createElement(X("positions"));
              DOMText*  positionsVal = rootdoc->createTextNode(X(tmpstr.c_str()));
              positionsElem->appendChild(positionsVal);
              patterndataElem->appendChild(positionsElem);
              patternElem->appendChild(patterndataElem);
            
              tmp=(*i)->getSecPat().getPat();
              tmpstr=biu::Util_String::intvec2str(tmp,":");   
              tmpstr.resize(tmpstr.length()-1);
            
              patterndataElem = rootdoc->createElement(X("pattern_data"));
              patterndataElem->setAttribute(X("molecule_ref"), X((*i)->getSecPat().getID().getMolRef().c_str()));
              patterndataElem->setAttribute(X("model_ref"), X((*i)->getSecPat().getID().getStrRef().c_str()));
              positionsElem = rootdoc->createElement(X("positions"));
              positionsVal = rootdoc->createTextNode(X(tmpstr.c_str()));
              positionsElem->appendChild(positionsVal);
              patterndataElem->appendChild(positionsElem);
              patternElem->appendChild(patterndataElem);
              
            rootElem->appendChild(patternElem);    
        }
     }
}


